import { Editor } from "@monaco-editor/react";
import { editor } from "monaco-editor";
import * as monaco from "monaco-editor/esm/vs/editor/editor.api";
import Tabs from "@ui/tabs";
import { LucideIcon } from "lucide-react";
import { useEffect, useRef, useState } from "react";
import { useStore } from "zustand";
import { themeStore } from "@/stores/theme.store";

export type CodeEditorFile = {
  uri: string;
  title?: string;
  icon?: LucideIcon;
  lang: "html" | "javascript" | "css";
  content: string;
  hidden?: boolean;
  pinned?: boolean;
};

export type CodeEditorRef = {
  editor: editor.IStandaloneCodeEditor;
  monaco: typeof monaco;
  getModels: () => editor.IModel[];
  loadFiles: (files: CodeEditorFile[]) => void;
  getData: () => {
    lang: string;
    value: string;
    fullUri: monaco.Uri;
    uri: string;
  }[];
  onSave: () => void;
  onFormat: () => void;
  clearPersistedData: () => void;
};

type CodeEditorProps = {
  files?: CodeEditorFile[];
  persistId?: string;
  onLoad?: (ref: CodeEditorRef) => void;
  showHiddenFiles?: boolean;
};

const CodeEditor = ({
  persistId,
  onLoad,
  showHiddenFiles,
  ...props
}: CodeEditorProps) => {
  const editorRef = useRef<editor.IStandaloneCodeEditor | null>(null);
  const monacoRef = useRef<typeof monaco | null>(null);

  const [files, setCurrentFiles] = useState<CodeEditorFile[]>([]);
  const [currentFile, setCurrentFile] = useState("");
  const [isEditorReady, setEditorReady] = useState(false);
  const isDarkMode = useStore(themeStore, (i) => i.isDarkMode);

  useEffect(() => {
    const editor = editorRef.current;
    const monaco = monacoRef.current;

    if (!editor || !monaco || !isEditorReady || !props.files?.length) {
      return;
    }

    loadFiles(props.files);
    onLoad?.({
      editor,
      monaco,
      getModels,
      loadFiles,
      getData,
      onSave,
      onFormat,
      clearPersistedData,
    });
  }, [isEditorReady, props.files, persistId]);

  useEffect(() => {
    if (!currentFile?.length || !isEditorReady) {
      return;
    }

    if (!files.find((i) => i.uri === currentFile)) {
      return;
    }

    const uri = monaco.Uri.parse(currentFile);
    const model = monacoRef.current?.editor.getModel(uri);
    if (model) {
      editorRef.current?.setModel(model);
    }
  }, [currentFile, isEditorReady, files]);

  const loadFiles = (files: CodeEditorFile[]) => {
    const monaco = monacoRef.current;
    if (!monaco) {
      return;
    }

    // Clear existing models
    getModels().forEach((model) => {
      model.dispose();
    });

    files.forEach((file) => {
      const uri = monaco.Uri.parse(file.uri);
      const exist = monaco.editor.getModel(uri);
      if (exist) {
        return;
      }

      let content = file.content;
      if (persistId) {
        content = localStorage.getItem(`${persistId}:${file.uri}`) || content;
      }

      // Create model
      monaco.editor.createModel(content, file.lang, uri);
    });

    const visibleFiles = files
      .filter((i) => !i.hidden || showHiddenFiles)
      .sort((a, b) => {
        return a.pinned && b.pinned ? 0 : a.pinned ? -1 : b.pinned ? 1 : 0;
      });
    setCurrentFiles(visibleFiles);

    // Open first model
    setCurrentFile(visibleFiles[0]?.uri || "");
  };

  const getModels = () => {
    return monacoRef?.current?.editor.getModels() || [];
  };

  const getData = () => {
    const models = getModels();

    return models.map((model) => {
      const lang = model.getLanguageId();
      const value = model.getValue();
      const fullUri = model.uri;
      const uri = model.uri.path.slice(1);

      return { lang, value, fullUri, uri };
    });
  };

  const onSave = () => {
    const editor = editorRef.current;
    if (!editor || !persistId) {
      return;
    }

    const data = getData();
    data.forEach((file) => {
      localStorage.setItem(`${persistId}:${file.uri}`, file.value);
    });
    return data;
  };

  const onFormat = () => {
    const editor = editorRef.current;
    if (editor) {
      editor?.getAction("editor.action.formatDocument")?.run();
    }
  };

  const clearPersistedData = () => {
    if (!persistId) {
      return;
    }

    const data = getData();
    data.forEach((file) => localStorage.removeItem(`${persistId}:${file.uri}`));
    loadFiles(props.files || []);
  };

  return (
    <>
      <Tabs
        hideIfOneTab
        tabs={files.map((file) => ({
          title: file.title || file.uri,
          icon: file.icon,
          key: file.uri,
        }))}
        current={currentFile}
        onChange={(key) => {
          setCurrentFile(key);
        }}
      />

      <div className="flex-1 overflow-hidden">
        <Editor
          height="100%"
          defaultLanguage="javascript"
          theme={isDarkMode ? "vs-dark" : "light"}
          onMount={(editor, monaco) => {
            editorRef.current = editor;
            monacoRef.current = monaco;
            setEditorReady(true);
          }}
          options={{
            minimap: { enabled: false },
            scrollBeyondLastLine: false,
            fontSize: 16,
            lineNumbersMinChars: 3,
            // lineNumbers: "off",
            showFoldingControls: "never",
            // codeLens: false,
            wordWrap: "on",
          }}
        />
      </div>
    </>
  );
};

export default CodeEditor;
