import { TestCase } from "@/lib/playground";
import { Decode, Hook } from "console-feed";
import { Message } from "console-feed/lib/definitions/Component";
import { Bug, ClipboardCheck, PanelsTopLeft } from "lucide-react";
import { useCallback, useEffect, useMemo, useRef, useState } from "react";
import PlaygroundTests from "./playground-tests";
import ConsoleLogs from "./console-logs";
import { TabView, TabViewItem } from "@ui/tabs";
import { useStore } from "zustand";
import { themeStore } from "@/stores/theme.store";

export type PlaygroundPanelsRef = {
  iframe: HTMLIFrameElement;
  setFrameData: (data: string) => void;
  clearLogs: () => void;
};

type PlaygroundPanelsProps = {
  tests?: TestCase[];
  onTestResult?: (id: string, result: any) => void;
  onLoad: (ctx: PlaygroundPanelsRef) => void;
  showPreview?: boolean;
  showLogs?: boolean;
};

const PlaygroundPanels = ({
  tests,
  onTestResult,
  onLoad,
  showPreview,
  showLogs,
}: PlaygroundPanelsProps) => {
  const iframeRef = useRef<HTMLIFrameElement | null>(null);
  const [frameLogs, setFrameLogs] = useState<Message[]>([]);
  const colorScheme = useStore(themeStore, (i) =>
    i.isDarkMode ? "dark" : "light"
  );

  const onFrameLoad = useCallback(
    (el: any) => {
      if (el.contentWindow.consoleFn) {
        Hook(el.contentWindow.consoleFn, (log) => {
          const data = Decode(log) as never;
          setFrameLogs((cur) => [...cur, data]);
        });
      }

      if (typeof el.contentWindow.loadScript === "function") {
        el.contentWindow.loadScript();
      }

      tests?.forEach((item) => {
        let result;
        try {
          result = el.contentWindow.eval(item.test);
        } catch (err) {
          console.error(err);
        } finally {
          onTestResult?.(item.id!, result);
        }
      });

      onLoad({
        iframe: el,
        setFrameData: (data: string) => {
          el.srcdoc = data;
        },
        clearLogs: () => {
          setFrameLogs([]);
        },
      });
    },
    [setFrameLogs, tests, onLoad]
  );

  const frameElement = useMemo(() => {
    return (
      <iframe
        ref={iframeRef}
        referrerPolicy="origin"
        srcDoc=""
        onLoad={() => {
          const el = iframeRef.current as any;
          if (el?.contentWindow) {
            onFrameLoad(el);
          }
        }}
      />
    );
  }, [onFrameLoad]);

  useEffect(() => {
    const frameDoc = iframeRef.current?.contentWindow?.document;
    if (frameDoc) {
      frameDoc
        .querySelector("meta[name='color-scheme']")
        ?.setAttribute("content", colorScheme);
    }
  }, [colorScheme]);

  const tabs = useMemo(() => {
    return [
      {
        title: "Pratinjau",
        key: "preview",
        icon: PanelsTopLeft,
        element: frameElement,
        show: showPreview === true,
      },
      {
        title: "Hasil",
        key: "tests",
        icon: ClipboardCheck,
        Component: ({ className }: any) => (
          <PlaygroundTests tests={tests || []} className={className} />
        ),
        enabled: tests && tests?.length > 0,
      },
      {
        title: "Log",
        key: "logs",
        icon: Bug,
        Component: ({ className }: any) => (
          <ConsoleLogs logs={frameLogs} className={className} />
        ),
        enabled: showLogs !== false,
      },
    ] satisfies TabViewItem[];
  }, [frameElement, frameLogs, tests, showPreview, showLogs]);

  return (
    <TabView
      tabs={tabs}
      className="rounded-none"
      contentClassName="flex-1 shrink-0"
      hideIfOneTab
    />
  );
};

export default PlaygroundPanels;
