import { Fragment } from "react";
import {
  Code,
  PlayCircle,
  Expand,
  Shrink,
  X,
  EllipsisVertical,
  RotateCw,
} from "lucide-react";
import { useCallback, useEffect, useRef, useState } from "react";
import Button from "@ui/button";
import { nanoid } from "nanoid";
import { cn } from "@/lib/utils";
import { frameTemplate, TestCase } from "../../lib/playground";
import CodeEditor, { CodeEditorFile, CodeEditorRef } from "./code-editor";
import PlaygroundPanels, { PlaygroundPanelsRef } from "./playground-panels";
import Dropdown from "@ui/dropdown";

type PlaygroundProps = {
  className?: string;
  files?: CodeEditorFile[];
  tests?: TestCase[];
  persistId?: string;
  showHiddenFiles?: boolean;
  showLogs?: boolean;
};

const Playground = ({
  className,
  files,
  persistId,
  showHiddenFiles,
  showLogs,
  ...props
}: PlaygroundProps) => {
  const panelsRef = useRef<PlaygroundPanelsRef | null>(null);
  const editorRef = useRef<CodeEditorRef | null>(null);

  const [tests, setTests] = useState<TestCase[]>([]);
  const [hasRun, setHasRun] = useState(false);

  const onEditorLoad = useCallback((ctx: CodeEditorRef) => {
    editorRef.current = ctx;
    const { monaco, editor } = ctx;

    editor.addAction({
      id: "run",
      label: "Run",
      keybindings: [
        monaco.KeyMod.CtrlCmd | monaco.KeyCode.Enter,
        monaco.KeyMod.CtrlCmd | monaco.KeyCode.Enter,
      ],
      run: onRun,
    });

    editor.addAction({
      id: "save",
      label: "Save",
      keybindings: [monaco.KeyMod.CtrlCmd | monaco.KeyCode.KeyS],
      run: onRun,
    });
  }, []);

  const onTestResult = useCallback(
    (id: string, result: any) => {
      const idx = tests.findIndex((i) => i.id === id);
      if (idx < 0) {
        return;
      }

      setTests((cur) => {
        const res = [...cur];
        res[idx] = { ...res[idx], result, isFinished: true };
        return res;
      });
    },
    [tests, setTests]
  );

  const onRun = () => {
    const panels = panelsRef.current;
    const editor = editorRef.current;

    if (!panels || !editor) {
      return;
    }

    let body = "",
      script = "",
      style = "";

    editor.getData().forEach((file) => {
      const value = file.value + "\n";

      switch (file.lang) {
        case "html":
          body += value;
          break;
        case "javascript":
          script += value;
          break;
        case "css":
          style += value;
          break;
      }
    });

    const newTests = props.tests?.map((i) => ({ ...i, id: nanoid() })) || [];
    const doc = frameTemplate({
      body,
      script,
      style,
      tests: newTests.map((i) => ({ id: i.id, test: i.test })),
    });

    setTests(newTests);
    setHasRun(true);
    panels.setFrameData(doc);
    panels.clearLogs();
    editor.onFormat();
    setTimeout(() => editor.onSave(), 100);
  };

  const onResetFiles = () => {
    if (window.confirm("Apakah anda yakin ingin mengembalikan data ke awal?")) {
      editorRef.current?.clearPersistedData();
    }
  };

  if (!files?.length) {
    return null;
  }

  return (
    <PlaygroundContainer className={className}>
      {(ctx) => (
        <Fragment>
          <div className="p-2 flex flex-row items-center">
            <Code size={28} className="hidden lg:block ml-4 mr-3" />
            <Button
              icon={X}
              variant="ghost"
              className="lg:hidden -ml-2 -mr-1"
              onClick={() => ctx.setOpen(false)}
            />

            <p className="font-medium text-lg flex-1 truncate">Playground</p>
            <Button
              onClick={onRun}
              icon={PlayCircle}
              title="Ctrl + Enter to Run"
            >
              Mulai
            </Button>
            <Dropdown
              trigger={
                <Button variant="ghost" size="icon" icon={EllipsisVertical} />
              }
            >
              <Dropdown.Item
                icon={ctx.isExtended ? Shrink : Expand}
                onClick={() => ctx.setExtended(!ctx.isExtended)}
                className="hidden lg:block"
                title="Ctrl + Alt + M"
              >
                Tampilan Penuh
              </Dropdown.Item>
              <Dropdown.Item icon={RotateCw} onClick={onResetFiles}>
                Reset Files
              </Dropdown.Item>
            </Dropdown>
          </div>

          <div
            className={cn(
              "flex-1 flex flex-col items-stretch overflow-hidden",
              ctx.isExtended && "sm:grid sm:grid-cols-12"
            )}
          >
            <div className="flex-1 flex flex-col overflow-hidden sm:col-span-6 lg:col-span-7">
              <CodeEditor
                files={files}
                persistId={persistId}
                showHiddenFiles={showHiddenFiles}
                onLoad={onEditorLoad}
              />
            </div>

            <div
              className={cn(
                "flex-1 flex flex-col overflow-hidden sm:col-span-6 lg:col-span-5 transition-all duration-1000 ease-in-out opacity-100 border-t",
                !hasRun && !ctx.isExtended
                  ? "max-h-0 opacity-0"
                  : "max-h-[100vh]",
                ctx.isExtended && "border-l border-t-0 max-h-none"
              )}
            >
              <PlaygroundPanels
                onLoad={(ref) => {
                  panelsRef.current = ref;
                }}
                tests={tests}
                onTestResult={onTestResult}
                showPreview={files?.find((i) => i.lang === "html") != null}
                showLogs={showLogs}
              />
            </div>
          </div>
        </Fragment>
      )}
    </PlaygroundContainer>
  );
};

type IPlaygroundContext = {
  isExtended: boolean;
  setExtended: (isExtended: boolean) => void;
  isOpen: boolean;
  setOpen: (isOpen: boolean) => void;
};

type PlaygroundContainerProps = {
  className?: string;
  children?: (ctx: IPlaygroundContext) => React.ReactNode;
};

const PlaygroundContainer = ({
  className,
  children,
}: PlaygroundContainerProps) => {
  const [isExtended, setExtended] = useState(false);
  const [isOpen, setOpen] = useState(false);

  useEffect(() => {
    const onMouseDown = (e: MouseEvent) => {
      if (!isOpen) {
        return;
      }

      const element = document.getElementById("playground");
      if (element && !element.contains(e.target as Node)) {
        setOpen(false);
      }
    };

    const onKeyUp = (e: KeyboardEvent) => {
      if (e.key === "Escape" && (isOpen || isExtended)) {
        setOpen(false);
        setExtended(false);
      }

      // ctrl+alt+m to go extended
      if (e.ctrlKey && e.altKey && e.key === "m") {
        setExtended(!isExtended);
      }
    };

    window.addEventListener("mousedown", onMouseDown);
    window.addEventListener("keyup", onKeyUp);

    return () => {
      window.removeEventListener("mousedown", onMouseDown);
      window.removeEventListener("keyup", onKeyUp);
    };
  }, [isOpen, isExtended]);

  const contextValues = {
    isExtended,
    setExtended,
    isOpen,
    setOpen,
  };

  return (
    <Fragment>
      <Button
        className="fixed bottom-4 right-4 sm:bottom-8 sm:right-8 lg:hidden z-[39]"
        onClick={() => setOpen(true)}
      >
        <Code />
        <p>Coba</p>
      </Button>

      <div
        id="playground"
        className={cn(
          "flex flex-col overflow-hidden playground bg-base-100 w-full max-w-[400px] h-full fixed right-0 border-l top-0 translate-x-full transition-transform z-40 lg:static lg:translate-x-0 lg:max-w-[33%] shadow-xl lg:shadow-none",
          className,
          isOpen && "translate-x-0",
          isExtended && "!flex !max-w-none !fixed inset-0"
        )}
      >
        {children?.(contextValues)}
      </div>
    </Fragment>
  );
};

export default Playground;
