import { cn } from "@/lib/utils";
import { LucideIcon } from "lucide-react";
import React, { useEffect, useMemo, useState } from "react";
import { Slot } from "@radix-ui/react-slot";

export type Tab = {
  title: string;
  key: string;
  icon?: LucideIcon;
};

type TabsProps = {
  tabs?: Tab[];
  current?: string;
  onChange?: (idx: string) => void;
  hideIfOneTab?: boolean;
  className?: string;
  tabClassName?: string;
};

const Tabs = ({
  tabs = [],
  current,
  onChange,
  hideIfOneTab,
  className,
  tabClassName,
}: TabsProps) => {
  if (tabs.length < 2 && hideIfOneTab) {
    return null;
  }

  return (
    <div role="tablist" className={cn("tabs tabs-boxed p-2", className)}>
      {tabs.map((tab) => (
        <button
          key={tab.key}
          type="button"
          role="tab"
          className={cn(
            "tab h-10 text-sm gap-2",
            tabClassName,
            tab.key === current && "tab-active !border-b-primary text-primary"
          )}
          onClick={() => onChange?.(tab.key)}
        >
          {tab.icon ? <tab.icon size={18} /> : null}
          {tab.title}
        </button>
      ))}
    </div>
  );
};

export type TabViewItem = Tab & {
  enabled?: boolean;
  show?: boolean;
  element?: React.ReactNode;
  Component?: React.ComponentType<any>;
};

type TabViewProps = Omit<TabsProps, "tabs"> & {
  tabs: TabViewItem[];
  contentClassName?: string;
};

export const TabView = ({ tabs, contentClassName, ...props }: TabViewProps) => {
  const [curTab, setCurTab] = useState("");
  const tabList = useMemo(() => {
    if (!tabs) {
      return [];
    }

    return tabs
      .filter((tab) => tab.enabled !== false)
      .map((tab) => {
        const content = tab.element ? (
          tab.element
        ) : tab.Component ? (
          <tab.Component />
        ) : null;

        return { ...tab, content };
      });
  }, [tabs]);

  const visibleTabs = tabList.filter((tab) => tab.show !== false);

  useEffect(() => {
    setCurTab(visibleTabs[0]?.key || "");
  }, [visibleTabs.length]);

  return (
    <>
      <Tabs
        tabs={visibleTabs}
        current={curTab}
        onChange={setCurTab}
        {...props}
      />

      {tabList.map((tab) => (
        <Slot
          key={tab.key}
          role="tabpanel"
          className={cn(
            contentClassName,
            tab.key !== curTab ? "hidden" : undefined
          )}
        >
          {tab.content}
        </Slot>
      ))}
    </>
  );
};

export default Tabs;
